{% extends "admin/base_site.html" %}

{% load static kmuhelper_tags %}

{% block title %}{{ title }} | KMUHelper{% endblock %}

{% block branding %}
    {% kmuhelper_branding %}
{% endblock %}

{% block extrastyle %}
    {{ block.super }}
    <link rel="stylesheet" type="text/css" href="{% static 'admin/kmuhelper/css/style.css' %}">
{% endblock %}

{% block extrahead %}
    {{ block.super }}
    {% include "kmuhelper/_includes/pwa_head.html" %}
{% endblock %}
